function [a,GoQai,m,GoQrgap,adiff] = AntennaGoQ_NewtonIt(a,Xe,Xm,F,gap0,adiff0,m0,abound1)
% Newton iteration to maximize G/Q by solution of the dual problem 
% min_a max_I |FI|^2/(I'*(a*Xe + (1-a)*Xm)*I) 
%
% Input parameters:
% a: initail value for a, 0<= a <= 1
% Xe, Xm: electric and magnetic reactance matrices
% F: far field matrix 
% gap0: stopping criteria for the gap between dual problem and G/Q 
% adiff0: stopping critera for the parameter a
% m0: maximal number of iterations
% abound1: start value if a < 0, (1-abound1) if a > 1
% 
% Iutput paramters:
% a: final value for a
% GoQai: final value for G/Q
% m: number of iterations
% GoQrgap: final value for the gap
% adiff: final value for the difference in a
%
% Mats Gustafsson 2010-15, http://www.eit.lth.se/staff/mats.gustafsson
%
% See 
% M. Gustafsson, D. Tayli, C. Ehrenborg, M. Cismasu, S. Nordebo
% Tutorial on antenna current optimization using MATLAB and CVX
% 2015
% and
% M. Gustafsson, S. Nordebo:  
% Optimal antenna currents for Q, superdirectivity, and radiation patterns using convex optimization
% IEEE Transactions on Antennas and Propagation, Vol. 61, No. 3, pp. 1109-1118, 2013.
% for additional information
%%

abound2 = 1-abound1;
m=0;
GoQrgap = 1;
adiff = 1;
while (GoQrgap>gap0 && adiff>adiff0 && m<m0)
    m = m+1;
%     disp(sprintf('a=%0.16f',a))
    [d,dp,db,an,GoQai,GoQdi,GoQrgap,posdefp] = dualGoQ(a,Xe,Xm,F);
    if posdefp
        msg = strcat('Not positive definite: p=',num2str(posdefp),' m=',num2str(m),' a=',num2str(a));
        disp(msg)
        break
    end
    if an>1
        if a<abound2
            if a<abound1
                an = 0.5;
            else
                an = abound2;
            end
        elseif dp>0
            an = (1+a)/2;
        else
            an = (a+ap)/2;
        end
    end
    if an<0
        if a>abound1
            if a>abound2
                an = 0.5;
            else
                an =abound1;
            end
        elseif dp<0
            an = a/2;
        else
            an = (a+ap)/2;
        end
    end
    ap = a;
    a = an;
    adiff = abs(an-ap);
end

end

function [d,dp,db,an,GoQa,GoQd,GoQrgap,p] = dualGoQ(a,Xe,Xm,F)
eta0 =  299792458*4e-7*pi;  % Wave impedance in vacuum

X = Xm-Xe;
Xa = a*Xe+(1-a)*Xm;
[R,p] = chol(Xa);
if p
    msg = 'Xa indef';
    disp(msg);
    d=0;dp=0;db=0;an=0;GoQa=0;GoQd=0;GoQrgap=0;
else
    y = R'\F';
    J = R\y;
    d = 1/real(F*J);
    I = -1i*d*J; % current
    XI=X*I;
    dp = -real(I'*XI);
    y = R'\XI;
    XaXI = R\y;
    db = 2*dp^2/d - real(2*XI'*XaXI);
    an = a-dp/db;
    
    P = abs(F*I)^2*4*pi/eta0;% n. radiation int.
    QoGe = real(I'*Xe*I)/P; % Qe/G
    QoGm = real(I'*Xm*I)/P; % Qm/G
    GoQa = 1/max(QoGe,QoGm); % G/Q
    GoQd = 1/(a*QoGe+(1-a)*QoGm); % dual G/Q
    GoQrgap = GoQd/GoQa-1;
end
end