% Use CVX to maximize G/Q for a strip dipole geometry
% see Sec. VII.B in
% M. Gustafsson, D. Tayli, C. Ehrenborg, M. Cismasu, S. Nordebo
% Tutorial on antenna current optimization using MATLAB and CVX
% 2015

clear all
k1=0.5*2*pi; k2=k1; kN=1; lx=1; ly=0.02; Nx=100; Ny=1; % half-a-wavelenth strip dipole
lib = '..\data';  % path to the data library
Calc_RX_matrices_rec(k1,k2,kN,lx,ly,Nx,Ny,lib);
% load the data files
load(strcat(lib,'/RowRX_rec_x=1_y=0p02_Nx=100_Ny=1_k=3p1416'));
% constructs the reactance matrices Xe, Xm from the computed data
[Xe,p] = RX_rec_sym2full(Xe11,Xe12,Xe22,bas.BxNN,bas.ByNN,meshp.txN,meshp.tyN,2);
[Xm,p] = RX_rec_sym2full(Xm11,Xm12,Xm22,bas.BxNN,bas.ByNN,meshp.txN,meshp.tyN,2);
[Rr,p] = RX_rec_sym2full(Rr11,Rr12,Rr22,bas.BxNN,bas.ByNN,meshp.txN,meshp.tyN,2);
if 1
    % compute the far-field matrix for
    evh = [1 0 0];
    rvh = [0 0 1];
    F = farfieldmatrix(kl,bas,meshp,evh,rvh);
else
    % compute the far-field matrix for spherical mode
    sphn = 6; % x-directed electric dipole 
    F = sphmodematrix(kl,bas,meshp,sphn);
end

% CVX code for maximization of G/Q
cvx_begin
   variable I(N) complex;     % current
   variable w;                % n. stored energy         
   minimize w        
   subject to
     quad_form(I,Xe) <= w;    % n. stored E energy
     quad_form(I,Xm) <= w;    % n. stored M energy
     F*I == -1i;              % far-field 
cvx_end
GoQ = 4*pi/(w*eta0)           % bound on G/Q

% plot the current density
figure(1); clf
x = linspace(-lx/2,lx/2,N+2);       % x coordinates
plot(x,real([0; I/dy; 0]),x,imag([0; I/dy; 0]))
title('Current density on the strip dipole')
xlabel('x/l_x')
ylabel('J_x')
legend('Re J_x','Im J_x')

% antenna parameters from the max. G/Q problem
P = abs(F*I)*abs(F*I)/2/eta0; % radiation intensity
Pr = real(I'*Rr*I)/2;         % radiated power
D = 4*pi*P/Pr                 % res. directivity
We = real(I'*Xe*I)/4/kl;      % stored E energy
Wm = real(I'*Xm*I)/4/kl;      % stored M energy
W = max(We,Wm);               % stored energy
Q = 2*kl*W/Pr                 % Q
Qe = 2*kl*We/Pr;              % Q electric
Qm = 2*kl*Wm/Pr;              % Q magnetic
